/* ejemplo_serial.c - ejemplo serial */
#include <stdio.h>
#include <limits.h>

#define NUMNOTAS 12

int  pitchClass(int MIDI);
int  normalizedPitchClass(int MIDI, int root);
void trasponer(int fila[], int trasp[], int t);
void invertir(int fila[], int invert[]);
void serialMatrix(int fila[], int matriz[NUMNOTAS][]);
void textoAmidi(char *archivo_texto, char *archivo_midi);

/* Para recorrer la matriz serial se utilizan los indices de la primera fila para seleccionar otra fila */
/* La fila seleccionada es una trasposicion de la primera fila y se van leyendo retrogradadas alternadamente */

int main(){

	int pc, i, j, k;
	int midi[NUMNOTAS] = {55, 58, 62, 66, 69, 72, 76, 80, 83, 85, 87, 89};
	int secuencia[NUMNOTAS];
	int matriz[NUMNOTAS][NUMNOTAS];

	/* Para escribir un archivo midi */
	int velocity = 127, root = 55;
	float tiempo = 1, semicorchea = 0.0625, duracion;
	FILE *fPtr; /* puntero a archivo */

		
	printf("Notas MIDI: \n");
	for (i = 0; i < NUMNOTAS; i++){
		printf("%2d\t", midi[i]);
	}
	printf("\n");
	
	printf("Pitch classes: \n");
	for (i = 0; i < NUMNOTAS; i++){
		printf("%2d\t", pitchClass(midi[i]));
	}
	printf("\n");

	printf("Normalized pitch classes: \n");
	for (i = 0; i < NUMNOTAS; i++){
		pc = normalizedPitchClass(midi[i], midi[0]);
		printf("%2d\t", pc); 
		secuencia[i] = pc;
	}
	printf("\n");
	printf("\n");
	
	
	serialMatrix(secuencia, matriz);
	
	printf("Matriz serial: \n");
	for (i = 0; i < NUMNOTAS; i++){
		for (j = 0; j < NUMNOTAS; j++){
			printf("%3d", matriz[i][j]);
		}
		printf("\n");
	}
	printf("\n");
	
	/* Generacion de la secuencia */
	if ((fPtr = fopen("temp.txt", "w")) == NULL)
		printf("No se pudo crear archivo\n");
	else{
		for (i = 0; i < NUMNOTAS; i++){
			/* se busca la fila cuyo primer elemento coincide con secuencia[i] */
			k=0;
			while ((k < NUMNOTAS) && (secuencia[i] != matriz[k][0])) 
				k++;
			/* la duracion de cada nota en semicorcheas se detemina por la serie de la columna i */	
			if (i % 2 == 0){ // Si i es par se lee la fila de indice secuencia[i] 
				for (j = 0; j < NUMNOTAS; j++){ 
					duracion = (matriz[j][i] + 1) * semicorchea;
					//duracion = (matriz[k][j] + 1) * semicorchea;
					printf("%2d (%2.f)\t", matriz[k][j], duracion / semicorchea); 
					fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, matriz[k][j] + root, velocity);
					tiempo += duracion;
				}
				printf("\n"); 
			}
			else { // Si i es impar se lee el retrogrado de la fila de indice secuencia[i] 
				for (j = 0; j < NUMNOTAS; j++){
					duracion = (matriz[j][i] + 1) * semicorchea;
					//duracion = (matriz[k][j] + 1) * semicorchea;
					printf("%2d (%2.f)\t", matriz[k][NUMNOTAS-1-j], duracion / semicorchea); 
					fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, matriz[k][NUMNOTAS-1-j] + root, velocity);
					tiempo += duracion;
				}
				printf("\n"); 
			}
		}
		fclose(fPtr);
		textoAmidi("temp.txt", "secuencia.mid");
	}
			
	return 0;
}

int pitchClass(int MIDI){
	
	return (MIDI % 12);
}

int normalizedPitchClass(int MIDI, int root){
	
	return ((MIDI - root) % 12);
}

void trasponer(int fila[], int trasp[], int t){
	int i;
	
	for (i = 0; i < NUMNOTAS; i++){
		trasp[i] = pitchClass((fila[i] + t));
	}
}

void invertir(int fila[], int invert[]){
	int i;
	
	for (i = 0; i < NUMNOTAS; i++){
		invert[i] = pitchClass((12 - fila[i]));
	}

}

void serialMatrix(int fila[], int matriz[][NUMNOTAS]){

	int i;
	int inv[NUMNOTAS], filaMatriz[NUMNOTAS];

	// inversion
	invertir(fila, inv);

	for (i = 0; i < NUMNOTAS; i++){
		trasponer(fila, matriz[i], inv[i]);
	}	
}

void textoAmidi(char *archivo_texto, char *archivo_midi){
     
     char comando[PATH_MAX] = "./text2midi ";
     
     /* Se construye el comando a ejecutar */
     strcat(comando, archivo_texto);
     strcat(comando, " ");
     strcat(comando, archivo_midi);

     /* Se hace una llamada al sistema */
     system(comando);
}
