/* ejemplo_serial.c - ejemplo serial */
#include <stdio.h>
#include <limits.h>

#define NUMNOTAS 12

int  pitchClass(int MIDI);
int  normalizedPitchClass(int MIDI, int root);
void trasponer(int fila[], int trasp[], int t);
void invertir(int fila[], int invert[]);
void serialMatrix(int fila[], int matriz[NUMNOTAS][]);

void textoAmidi(char *archivo_texto_origen, char *archivo_midi, int numVoces, int programas[]);
void header(int, int, int, int, FILE *fPtr);
void endFile(FILE *fPtr);
void startTrack(int, int, int, int, FILE *fPtr);
void endTrack(int, int, FILE *fPtr);
void note(int, int, int, int, int, int, FILE *fPtr);

/* Para recorrer la matriz serial se utilizan los indices de la primera fila para seleccionar otra fila */
/* La fila seleccionada es una trasposicion de la primera fila y se van leyendo retrogradadas alternadamente */

int main(){

	int pc, i, j, k;
	int midi[NUMNOTAS] = {55, 58, 62, 66, 69, 72, 76, 80, 83, 85, 87, 89};
	int secuencia[NUMNOTAS];
	int secuenciaInv[NUMNOTAS];
	int matriz[NUMNOTAS][NUMNOTAS];

	/* Para escribir un archivo midi */
	int velocity = 127, root = 55, voz, programas[] = {11, 23};
	int tiempo = 0, semicorchea = 120, duracion;
	FILE *fPtr; /* puntero a archivo */

		
	printf("Notas MIDI: \n");
	for (i = 0; i < NUMNOTAS; i++){
		printf("%2d\t", midi[i]);
	}
	printf("\n");
	
	printf("Pitch classes: \n");
	for (i = 0; i < NUMNOTAS; i++){
		printf("%2d\t", pitchClass(midi[i]));
	}
	printf("\n");

	printf("Normalized pitch classes: \n");
	for (i = 0; i < NUMNOTAS; i++){
		pc = normalizedPitchClass(midi[i], midi[0]);
		printf("%2d\t", pc); 
		secuencia[i] = pc;
	}
	printf("\n");
	printf("\n");
	
	
	serialMatrix(secuencia, matriz);
	invertir(secuencia, secuenciaInv);
	
	printf("Secuencia invertida: \n");
	for (i = 0; i < NUMNOTAS; i++){
		printf("%2d\t", secuenciaInv[i]); 
	}
	printf("\n");

	printf("Matriz serial: \n");
	for (i = 0; i < NUMNOTAS; i++){
		for (j = 0; j < NUMNOTAS; j++){
			printf("%3d", matriz[i][j]);
		}
		printf("\n");
	}
	printf("\n");
	
	/* Generacion de la secuencia */
	if ((fPtr = fopen("temp.txt", "w")) == NULL)
		printf("No se pudo crear archivo\n");
	else{
		voz = 1; /* se repite el proceso para 2 voces, 1era voz */
		velocity = 127;
		printf("Voz %d \n", voz);
		for (i = 0; i < NUMNOTAS; i++){
			/* se busca la fila cuyo primer elemento coincide con secuencia[i] */
			k=0;
			while ((k < NUMNOTAS) && (secuencia[i] != matriz[k][0])) 
				k++;
			/* la duracion de cada nota en semicorcheas se detemina por la serie de la columna i */	
			if (i % 2 == 0){ // Si i es par se lee la fila de indice secuencia[i] 
				for (j = 0; j < NUMNOTAS; j++){ 
					duracion = (matriz[j][i] + 1) * semicorchea;
					printf("%2d (%2d)\t", matriz[k][j], matriz[j][i] + 1); 
					fprintf(fPtr, "%d\t%d\t%d\t%d\t%d\n", voz, tiempo, duracion, matriz[k][j] + root, velocity);
					tiempo += duracion;
				}
				printf("\n"); 
			}
			else { // Si i es impar se lee el retrogrado de la fila de indice secuencia[i] 
				for (j = 0; j < NUMNOTAS; j++){
					duracion = (matriz[j][i] + 1) * semicorchea;
					printf("%2d (%2d)\t", matriz[k][NUMNOTAS-1-j], matriz[j][i] + 1); 
					fprintf(fPtr, "%d\t%d\t%d\t%d\t%d\n", voz, tiempo, duracion, matriz[k][NUMNOTAS-1-j] + root, velocity);
					tiempo += duracion;
				}
				printf("\n"); 
			}
		}
		voz = 2; /* se repite el proceso para 2 voces, 2da voz */
		tiempo = 0; /* se reinicia el tiempo para la segunda voz */
		velocity = 80;
		printf("Voz %d \n", voz);
		for (i = 0; i < NUMNOTAS; i++){
			/* se busca la columna cuyo primer elemento coincide con secuencia[i] */
			k=0;
			while ((k < NUMNOTAS) && (secuenciaInv[i] != matriz[0][k])) 
				k++;
			/* la duracion de cada nota en semicorcheas se detemina por la serie de la fila i */	
			if (i % 2 == 0){ // Si i es par se lee la columna de indice secuenciaInv[i] 
				for (j = 0; j < NUMNOTAS; j++){ 
					duracion = (matriz[i][j] + 1) * semicorchea;
					printf("%2d (%2d)\t", matriz[j][k], matriz[i][j] + 1); 
					fprintf(fPtr, "%d\t%d\t%d\t%d\t%d\n", voz, tiempo, duracion, matriz[j][k] + root, velocity);
					tiempo += duracion;
				}
				printf("\n"); 
			}
			else { // Si i es impar se lee el retrogrado de la columna de indice secuencia[i] 
				for (j = 0; j < NUMNOTAS; j++){
					duracion = (matriz[i][j] + 1) * semicorchea;
					printf("%2d (%2d)\t", matriz[NUMNOTAS-1-j][k], matriz[i][j] + 1); 
					fprintf(fPtr, "%d\t%d\t%d\t%d\t%d\n", voz, tiempo, duracion, matriz[NUMNOTAS-1-j][k] + root, velocity);
					tiempo += duracion;
				}
					printf("\n"); 
			}
		}

		fclose(fPtr);
		textoAmidi("temp.txt", "secuencia.mid", voz, programas);
	}
			
	return 0;
}

int pitchClass(int MIDI){
	
	return (MIDI % 12);
}

int normalizedPitchClass(int MIDI, int root){
	
	return ((MIDI - root) % 12);
}

void trasponer(int fila[], int trasp[], int t){
	int i;
	
	for (i = 0; i < NUMNOTAS; i++){
		trasp[i] = pitchClass((fila[i] + t));
	}
}

void invertir(int fila[], int invert[]){
	int i;
	
	for (i = 0; i < NUMNOTAS; i++){
		invert[i] = pitchClass((12 - fila[i]));
	}

}

void serialMatrix(int fila[], int matriz[][NUMNOTAS]){

	int i;
	int inv[NUMNOTAS], filaMatriz[NUMNOTAS];

	// inversion
	invertir(fila, inv);

	for (i = 0; i < NUMNOTAS; i++){
		trasponer(fila, matriz[i], inv[i]);
	}	
}

void textoAmidi(char *archivo_texto_origen, char *archivo_midi, int numVoces, int programas[]){
     
     /* Se escribe un archivo de texto en formato similar a MIDI */
     /* Luego ese archivo de texto se convierte a MIDI usando csvmidi */
     int bpm = 120, tempo =  60000000/bpm, division = 480; /* Header */
     int format = 1, nTracks = numVoces; /* Numero de canales igual a numero de voces */ 
     int numTrack = 1, channel = 0;
     int timeStart = 0, timeEnd = 0;
     //float tiempoTrack = 0;
     int tiempoTrack = 0;
     char comando[PATH_MAX] = "./csvmidi ";
     char *archivo_texto_destino = "temp2.txt";
     FILE *fPtrDestino, *fPtrOrigen;
     int i, voz, notaMIDI, intensidad, duracion, tiempo; /* parametros de cada nota */
     
     if ((fPtrDestino = fopen(archivo_texto_destino, "w")) == NULL)
	printf("No se pudo crear archivo\n");
     else{
	header(format, ++nTracks, division, tempo, fPtrDestino);
	
	if ((fPtrOrigen = fopen(archivo_texto_origen, "r")) == NULL)
		printf("No se pudo abrir el archivo\n");
	else{
	  /* Se lee el archivo de texto generando un canal para cada una de las voces */
	  for (i = numVoces; i > 0; i--){ 
		startTrack(++numTrack, timeStart, ++channel, programas[i-1], fPtrDestino);
		/* se leen de a una las lineas */
		do{
		  fscanf(fPtrOrigen, "%d %d %d %d %d\n", &voz, &tiempo, &duracion, &notaMIDI, &intensidad);
		  /* si la linea contiene datos sobre la voz actual */
		  if (voz == i){
			note(numTrack, tiempo, channel, duracion, notaMIDI, intensidad, fPtrDestino);
			tiempoTrack = tiempo + duracion;
		  }
		}while (!feof(fPtrOrigen));
		/* voy al comienzo del archivo */
		rewind(fPtrOrigen);
		endTrack(numTrack, tiempoTrack, fPtrDestino);
	  }
	}  
	
	endFile(fPtrDestino);
	fclose(fPtrDestino);
	
	/* Se convierte el archivo de texto en un archivo MIDI */
	/* Se construye el comando a ejecutar */
	strcat(comando, archivo_texto_destino);
	strcat(comando, " ");
	strcat(comando, archivo_midi);
	
//	printf("%s\n", comando);
	
	/* Se hace una llamada al sistema */
	system(comando);
     }
}

/* Funciones para escribir a MIDI */

void header(int format, int nTracks, int division, int tempo, FILE *fPtr){

	fprintf(fPtr, "0, 0, Header, %d, %d, %d\n", format, nTracks, division);
	fprintf(fPtr, "1, 0, Start_track\n");
	fprintf(fPtr, "1, 0, Tempo, %d\n", tempo);
	fprintf(fPtr, "1, 0, End_track\n");

}

void endFile(FILE *fPtr){
	fprintf(fPtr, "0, 0, End_of_file\n");
}

void startTrack(int numTrack, int timeStart, int channel, int program, FILE *fPtr){

	fprintf(fPtr, "%d, %d, Start_track\n", numTrack, timeStart);
	fprintf(fPtr, "%d, %d, Program_c, %d, %d\n", numTrack, timeStart, channel, program);

}

void endTrack(int numTrack, int timeEnd, FILE *fPtr){

	fprintf(fPtr, "%d, %d, End_track\n", numTrack, timeEnd);

}

void note(int numTrack, int timeOn, int channel, int dur, int keyNum, int velocity, FILE *fPtr){

	fprintf(fPtr, "%d, %d, Note_on_c, %d, %d, %d\n", numTrack, timeOn, channel, keyNum, velocity);
	fprintf(fPtr, "%d, %d, Note_off_c, %d, %d, %d\n", numTrack, timeOn+dur, channel, keyNum, 0);

}
