/* serial_operations.c - Serial operations */
#include <stdio.h>
#include <limits.h>

#define NUMNOTAS 12

int  pitchClass(int MIDI);
int  normalizedPitchClass(int MIDI, int root);
void retrogradar(int fila[], int retro[]);
void trasponer(int fila[], int trasp[], int t);
void invertir(int fila[], int invert[]);
void retrogradarInvertir(int fila[], int reinv[]);
void textoAmidi(char *archivo_texto, char *archivo_midi);

int main(){

	int pc, i;
	int midi[NUMNOTAS] = {55, 58, 62, 66, 69, 72, 76, 80, 83, 85, 87, 89};
	int secuencia[NUMNOTAS], secuenciaRetro[NUMNOTAS], secuenciaTrasp[NUMNOTAS];
	int secuenciaInv[NUMNOTAS], secuenciaReInv[NUMNOTAS];
	/* Para escribir un archivo midi */
	int velocity = 127, root = 55;
	float tiempo = 1, duracion = 0.25;
	FILE *fPtr; /* puntero a archivo */

	printf("Notas MIDI: \n");
	for (i = 0; i < NUMNOTAS; i++){
		printf("%2d\t", midi[i]);
	}
	printf("\n");
	
	printf("Pitch classes: \n");
	for (i = 0; i < NUMNOTAS; i++){
		printf("%2d\t", pitchClass(midi[i]));
	}
	printf("\n");

	printf("Normalized pitch classes: \n");
	for (i = 0; i < NUMNOTAS; i++){
		pc = normalizedPitchClass(midi[i], midi[0]);
		printf("%2d\t", pc); 
		secuencia[i] = pc;
	}
	printf("\n");
	
	
	// Operaciones:
	retrogradar(secuencia, secuenciaRetro);
	trasponer(secuencia, secuenciaTrasp, 3);
	invertir(secuencia, secuenciaInv);
	retrogradarInvertir(secuencia, secuenciaReInv);

	if ((fPtr = fopen("temp.txt", "w")) == NULL)
		printf("No se pudo crear archivo\n");
	else{

		for (i = 0; i < NUMNOTAS; i++){
			fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, secuencia[i] + root, velocity);
			tiempo += duracion;
		}
		tiempo += 1;
		printf("Secuencia retrogradada: \n");
		for (i = 0; i < NUMNOTAS; i++){
			printf("%2d\t", secuenciaRetro[i]); 
			fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, secuenciaRetro[i] + root, velocity);
			tiempo += duracion;
		}
		tiempo += 1;
		printf("\n");
		printf("Secuencia traspuesta: \n");
		for (i = 0; i < NUMNOTAS; i++){
			printf("%2d\t", secuenciaTrasp[i]); 
			fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, secuenciaTrasp[i] + root, velocity);
			tiempo += duracion;
		}
		tiempo += 1;
		printf("\n");
		printf("Secuencia invertida: \n");
		for (i = 0; i < NUMNOTAS; i++){
			printf("%2d\t", secuenciaInv[i]); 
			fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, secuenciaInv[i] + root, velocity);
			tiempo += duracion;
		}
		tiempo += 1;
		printf("\n");
		printf("Secuencia retrogradada invertida: \n");
		for (i = 0; i < NUMNOTAS; i++){
			printf("%2d\t", secuenciaReInv[i]); 
			fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, secuenciaReInv[i] + root, velocity);
			tiempo += duracion;
		}
		printf("\n");
		
		fclose(fPtr);
		textoAmidi("temp.txt", "secuencia.mid");
	}
	
	return 0;
}

int pitchClass(int MIDI){
	
	return (MIDI % 12);
}

int normalizedPitchClass(int MIDI, int root){
	
	return ((MIDI - root) % 12);
}

void retrogradar(int fila[], int retro[]){
	int i;
	
	for (i = 0; i < NUMNOTAS; i++){
		retro[NUMNOTAS-i-1] = fila[i];
	}
}

void trasponer(int fila[], int trasp[], int t){
	int i;
	
	for (i = 0; i < NUMNOTAS; i++){
		trasp[i] = pitchClass((fila[i] + t));
	}
}

void invertir(int fila[], int invert[]){
	int i;
	
	for (i = 0; i < NUMNOTAS; i++){
		invert[i] = pitchClass((12 - fila[i]));
	}

}

void retrogradarInvertir(int fila[], int reinv[]){

	int retro[NUMNOTAS];

	retrogradar(fila, retro);
	invertir(retro, reinv);
}

void textoAmidi(char *archivo_texto, char *archivo_midi){
     
     char comando[PATH_MAX] = "./text2midi ";
     
     /* Se construye el comando a ejecutar */
     strcat(comando, archivo_texto);
     strcat(comando, " ");
     strcat(comando, archivo_midi);

     /* Se hace una llamada al sistema */
     system(comando);
}
